@@ -7,12 +7,14 @@
 mod access;
 mod delete;
 mod empty;
+mod entry;
 pub mod exec;
 pub mod fs;
 mod glob;
 mod group;
 mod lname;
 mod logical_matchers;
+mod ls;
 mod name;
 mod path;
 mod perm;
@@ -25,17 +27,18 @@ mod samefile;
 mod size;
 #[cfg(unix)]
 mod stat;
-mod time;
+pub mod time;
 mod type_matcher;
 mod user;
 
 use ::regex::Regex;
 use chrono::{DateTime, Datelike, NaiveDateTime, Utc};
 use fs::FileSystemMatcher;
+use ls::Ls;
+use std::fs::{File, Metadata};
 use std::path::Path;
 use std::time::SystemTime;
 use std::{error::Error, str::FromStr};
-use walkdir::DirEntry;
 
 use self::access::AccessMatcher;
 use self::delete::DeleteMatcher;
@@ -62,25 +65,91 @@ use self::time::{
     FileAgeRangeMatcher, FileTimeMatcher, FileTimeType, NewerMatcher, NewerOptionMatcher,
     NewerOptionType, NewerTimeMatcher,
 };
-use self::type_matcher::TypeMatcher;
+use self::type_matcher::{TypeMatcher, XtypeMatcher};
 use self::user::{NoUserMatcher, UserMatcher};
 
 use super::{Config, Dependencies};
 
+pub use entry::{FileType, WalkEntry, WalkError};
+
+/// Symlink following mode.
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
+pub enum Follow {
+    /// Never follow symlinks (-P; default).
+    Never,
+    /// Follow symlinks on root paths only (-H).
+    Roots,
+    /// Always follow symlinks (-L).
+    Always,
+}
+
+impl Follow {
+    /// Check whether to follow a path of the given depth.
+    pub fn follow_at_depth(self, depth: usize) -> bool {
+        match self {
+            Follow::Never => false,
+            Follow::Roots => depth == 0,
+            Follow::Always => true,
+        }
+    }
+
+    /// Get metadata for a [WalkEntry].
+    pub fn metadata(self, entry: &WalkEntry) -> Result<Metadata, WalkError> {
+        if self.follow_at_depth(entry.depth()) == entry.follow() {
+            // Same follow flag, re-use cached metadata
+            entry.metadata().cloned()
+        } else if !entry.follow() && !entry.file_type().is_symlink() {
+            // Not a symlink, re-use cached metadata
+            entry.metadata().cloned()
+        } else if entry.follow() && entry.file_type().is_symlink() {
+            // Broken symlink, re-use cached metadata
+            entry.metadata().cloned()
+        } else {
+            self.metadata_at_depth(entry.path(), entry.depth())
+        }
+    }
+
+    /// Get metadata for a path from the command line.
+    pub fn root_metadata(self, path: impl AsRef<Path>) -> Result<Metadata, WalkError> {
+        self.metadata_at_depth(path, 0)
+    }
+
+    /// Get metadata for a path, following symlinks as necessary.
+    pub fn metadata_at_depth(
+        self,
+        path: impl AsRef<Path>,
+        depth: usize,
+    ) -> Result<Metadata, WalkError> {
+        let path = path.as_ref();
+
+        if self.follow_at_depth(depth) {
+            match path.metadata().map_err(WalkError::from) {
+                Ok(meta) => return Ok(meta),
+                Err(e) if !e.is_not_found() => return Err(e),
+                _ => {}
+            }
+        }
+
+        Ok(path.symlink_metadata()?)
+    }
+}
+
 /// Struct holding references to outputs and any inputs that can't be derived
 /// from the file/directory info.
 pub struct MatcherIO<'a> {
     should_skip_dir: bool,
+    exit_code: i32,
     quit: bool,
-    deps: &'a dyn Dependencies<'a>,
+    deps: &'a dyn Dependencies,
 }
 
 impl<'a> MatcherIO<'a> {
-    pub fn new(deps: &'a dyn Dependencies<'a>) -> MatcherIO<'a> {
+    pub fn new(deps: &dyn Dependencies) -> MatcherIO<'_> {
         MatcherIO {
-            deps,
             should_skip_dir: false,
+            exit_code: 0,
             quit: false,
+            deps,
         }
     }
 
@@ -93,6 +162,15 @@ impl<'a> MatcherIO<'a> {
         self.should_skip_dir
     }
 
+    pub fn set_exit_code(&mut self, code: i32) {
+        self.exit_code = code;
+    }
+
+    #[must_use]
+    pub fn exit_code(&self) -> i32 {
+        self.exit_code
+    }
+
     pub fn quit(&mut self) {
         self.quit = true;
     }
@@ -122,7 +200,7 @@ pub trait Matcher: 'static {
     }
 
     /// Returns whether the given file matches the object's predicate.
-    fn matches(&self, file_info: &DirEntry, matcher_io: &mut MatcherIO) -> bool;
+    fn matches(&self, entry: &WalkEntry, matcher_io: &mut MatcherIO) -> bool;
 
     /// Returns whether the matcher has any side-effects (e.g. executing a
     /// command, deleting a file). Iff no such matcher exists in the chain, then
@@ -148,8 +226,8 @@ impl Matcher for Box<dyn Matcher> {
         self
     }
 
-    fn matches(&self, file_info: &DirEntry, matcher_io: &mut MatcherIO) -> bool {
-        (**self).matches(file_info, matcher_io)
+    fn matches(&self, entry: &WalkEntry, matcher_io: &mut MatcherIO) -> bool {
+        (**self).matches(entry, matcher_io)
     }
 
     fn has_side_effects(&self) -> bool {
@@ -202,7 +280,7 @@ pub fn build_top_level_matcher(
     if !top_level_matcher.has_side_effects() {
         let mut new_and_matcher = AndMatcherBuilder::new();
         new_and_matcher.new_and_condition(top_level_matcher);
-        new_and_matcher.new_and_condition(Printer::new(PrintDelimiter::Newline));
+        new_and_matcher.new_and_condition(Printer::new(PrintDelimiter::Newline, None));
         return Ok(new_and_matcher.build());
     }
     Ok(top_level_matcher)
@@ -339,6 +417,13 @@ fn parse_str_to_newer_args(input: &str) -> Option<(String, String)> {
     }
 }
 
+/// Creates a file if it doesn't exist.
+/// If it does exist, it will be overwritten.
+fn get_or_create_file(path: &str) -> Result<File, Box<dyn Error>> {
+    let file = File::create(path)?;
+    Ok(file)
+}
+
 /// The main "translate command-line args into a matcher" function. Will call
 /// itself recursively if it encounters an opening bracket. A successful return
 /// consists of a tuple containing the new index into the args array to use (if
@@ -347,7 +432,7 @@ fn build_matcher_tree(
     args: &[&str],
     config: &mut Config,
     arg_index: usize,
-    expecting_bracket: bool,
+    mut expecting_bracket: bool,
 ) -> Result<(usize, Box<dyn Matcher>), Box<dyn Error>> {
     let mut top_level_matcher = ListMatcherBuilder::new();
 
@@ -361,8 +446,8 @@ fn build_matcher_tree(
     let mut invert_next_matcher = false;
     while i < args.len() {
         let possible_submatcher = match args[i] {
-            "-print" => Some(Printer::new(PrintDelimiter::Newline).into_box()),
-            "-print0" => Some(Printer::new(PrintDelimiter::Null).into_box()),
+            "-print" => Some(Printer::new(PrintDelimiter::Newline, None).into_box()),
+            "-print0" => Some(Printer::new(PrintDelimiter::Null, None).into_box()),
             "-printf" => {
                 if i >= args.len() - 1 {
                     return Err(From::from(format!("missing argument to {}", args[i])));
@@ -370,6 +455,25 @@ fn build_matcher_tree(
                 i += 1;
                 Some(Printf::new(args[i])?.into_box())
             }
+            "-fprint" => {
+                if i >= args.len() - 1 {
+                    return Err(From::from(format!("missing argument to {}", args[i])));
+                }
+                i += 1;
+
+                let file = get_or_create_file(args[i])?;
+                Some(Printer::new(PrintDelimiter::Newline, Some(file)).into_box())
+            }
+            "-ls" => Some(Ls::new(None).into_box()),
+            "-fls" => {
+                if i >= args.len() - 1 {
+                    return Err(From::from(format!("missing argument to {}", args[i])));
+                }
+                i += 1;
+
+                let file = get_or_create_file(args[i])?;
+                Some(Ls::new(Some(file)).into_box())
+            }
             "-true" => Some(TrueMatcher.into_box()),
             "-false" => Some(FalseMatcher.into_box()),
             "-lname" | "-ilname" => {
@@ -400,7 +504,7 @@ fn build_matcher_tree(
                 }
                 i += 1;
                 regex_type = regex::RegexType::from_str(args[i])?;
-                None
+                Some(TrueMatcher.into_box())
             }
             "-regex" => {
                 if i >= args.len() - 1 {
@@ -423,6 +527,13 @@ fn build_matcher_tree(
                 i += 1;
                 Some(TypeMatcher::new(args[i])?.into_box())
             }
+            "-xtype" => {
+                if i >= args.len() - 1 {
+                    return Err(From::from(format!("missing argument to {}", args[i])));
+                }
+                i += 1;
+                Some(XtypeMatcher::new(args[i])?.into_box())
+            }
             "-fstype" => {
                 if i >= args.len() - 1 {
                     return Err(From::from(format!("missing argument to {}", args[i])));
@@ -440,7 +551,7 @@ fn build_matcher_tree(
                     return Err(From::from(format!("missing argument to {}", args[i])));
                 }
                 i += 1;
-                Some(NewerMatcher::new(args[i])?.into_box())
+                Some(NewerMatcher::new(args[i], config.follow)?.into_box())
             }
             "-mtime" | "-atime" | "-ctime" => {
                 if i >= args.len() - 1 {
@@ -448,7 +559,7 @@ fn build_matcher_tree(
                 }
                 let file_time_type = match args[i] {
                     "-atime" => FileTimeType::Accessed,
-                    "-ctime" => FileTimeType::Created,
+                    "-ctime" => FileTimeType::Changed,
                     "-mtime" => FileTimeType::Modified,
                     // This shouldn't be possible. We've already checked the value
                     // is one of those three values.
@@ -456,7 +567,7 @@ fn build_matcher_tree(
                 };
                 let days = convert_arg_to_comparable_value(args[i], args[i + 1])?;
                 i += 1;
-                Some(FileTimeMatcher::new(file_time_type, days).into_box())
+                Some(FileTimeMatcher::new(file_time_type, days, config.today_start).into_box())
             }
             "-amin" | "-cmin" | "-mmin" => {
                 if i >= args.len() - 1 {
@@ -464,13 +575,16 @@ fn build_matcher_tree(
                 }
                 let file_time_type = match args[i] {
                     "-amin" => FileTimeType::Accessed,
-                    "-cmin" => FileTimeType::Created,
+                    "-cmin" => FileTimeType::Changed,
                     "-mmin" => FileTimeType::Modified,
                     _ => unreachable!("Encountered unexpected value {}", args[i]),
                 };
                 let minutes = convert_arg_to_comparable_value(args[i], args[i + 1])?;
                 i += 1;
-                Some(FileAgeRangeMatcher::new(file_time_type, minutes).into_box())
+                Some(
+                    FileAgeRangeMatcher::new(file_time_type, minutes, config.today_start)
+                        .into_box(),
+                )
             }
             "-size" => {
                 if i >= args.len() - 1 {
@@ -542,7 +656,8 @@ fn build_matcher_tree(
                 }
                 i += 1;
                 let path = args[i];
-                let matcher = SameFileMatcher::new(path).map_err(|e| format!("{path}: {e}"))?;
+                let matcher = SameFileMatcher::new(path, config.follow)
+                    .map_err(|e| format!("{path}: {e}"))?;
                 Some(matcher.into_box())
             }
             "-user" => {
@@ -692,20 +807,44 @@ fn build_matcher_tree(
 
                 return Ok((i, top_level_matcher.build()));
             }
+            "-follow" => {
+                // This option affects multiple matchers.
+                // 1. It will use noleaf by default. (but -noleaf No change of behavior)
+                // Unless -L or -H is specified:
+                // 2. changes the behaviour of the -newer predicate.
+                // 3. consideration applies to -newerXY, -anewer and -cnewer
+                // 4. -type predicate will always match against the type of
+                //    the file that a symbolic link points to rather than the link itself.
+                //
+                // 5. causes the -lname and -ilname predicates always to return false.
+                //    (unless they happen to match broken symbolic links)
+                config.follow = Follow::Always;
+                config.no_leaf_dirs = true;
+                Some(TrueMatcher.into_box())
+            }
+            "-daystart" => {
+                config.today_start = true;
+                Some(TrueMatcher.into_box())
+            }
+            "-noleaf" => {
+                // No change of behavior
+                config.no_leaf_dirs = true;
+                Some(TrueMatcher.into_box())
+            }
             "-d" | "-depth" => {
                 // TODO add warning if it appears after actual testing criterion
                 config.depth_first = true;
-                None
+                Some(TrueMatcher.into_box())
             }
             "-mount" | "-xdev" => {
                 // TODO add warning if it appears after actual testing criterion
                 config.same_file_system = true;
-                None
+                Some(TrueMatcher.into_box())
             }
             "-sorted" => {
                 // TODO add warning if it appears after actual testing criterion
                 config.sorted_output = true;
-                None
+                Some(TrueMatcher.into_box())
             }
             "-maxdepth" => {
                 if i >= args.len() - 1 {
@@ -713,7 +852,7 @@ fn build_matcher_tree(
                 }
                 config.max_depth = convert_arg_to_number(args[i], args[i + 1])?;
                 i += 1;
-                None
+                Some(TrueMatcher.into_box())
             }
             "-mindepth" => {
                 if i >= args.len() - 1 {
@@ -721,7 +860,7 @@ fn build_matcher_tree(
                 }
                 config.min_depth = convert_arg_to_number(args[i], args[i + 1])?;
                 i += 1;
-                None
+                Some(TrueMatcher.into_box())
             }
             "-help" | "--help" => {
                 config.help_requested = true;
@@ -767,6 +906,12 @@ fn build_matcher_tree(
                 }
             }
         };
+        i += 1;
+        if config.help_requested || config.version_requested {
+            // Ignore anything, even invalid expressions, after -help/-version
+            expecting_bracket = false;
+            break;
+        }
         if let Some(submatcher) = possible_submatcher {
             if invert_next_matcher {
                 top_level_matcher.new_and_condition(NotMatcher::new(submatcher));
@@ -775,7 +920,6 @@ fn build_matcher_tree(
                 top_level_matcher.new_and_condition(submatcher);
             }
         }
-        i += 1;
     }
     if expecting_bracket {
         return Err(From::from(
@@ -791,25 +935,28 @@ mod tests {
     use super::*;
     use crate::find::tests::fix_up_slashes;
     use crate::find::tests::FakeDependencies;
-    use walkdir::WalkDir;
 
-    /// Helper function for tests to get a `DirEntry` object. directory should
+    /// Helper function for tests to get a [WalkEntry] object. root should
     /// probably be a string starting with `test_data/` (cargo's tests run with
     /// a working directory set to the root findutils folder).
-    pub fn get_dir_entry_for(directory: &str, filename: &str) -> DirEntry {
-        for wrapped_dir_entry in WalkDir::new(fix_up_slashes(directory)) {
-            let dir_entry = wrapped_dir_entry.unwrap();
-            if dir_entry
-                .path()
-                .strip_prefix(directory)
-                .unwrap()
-                .to_string_lossy()
-                == fix_up_slashes(filename)
-            {
-                return dir_entry;
-            }
-        }
-        panic!("Couldn't find {filename} in {directory}");
+    pub fn get_dir_entry_for(root: &str, path: &str) -> WalkEntry {
+        get_dir_entry_follow(root, path, Follow::Never)
+    }
+
+    /// Get a [WalkEntry] with an explicit [Follow] flag.
+    pub fn get_dir_entry_follow(root: &str, path: &str, follow: Follow) -> WalkEntry {
+        let root = fix_up_slashes(root);
+        let root = Path::new(&root);
+
+        let path = fix_up_slashes(path);
+        let path = if path.is_empty() {
+            root.to_owned()
+        } else {
+            root.join(path)
+        };
+
+        let depth = path.components().count() - root.components().count();
+        WalkEntry::new(path, depth, follow)
     }
 
     #[test]
@@ -1170,6 +1317,15 @@ mod tests {
         }
     }
 
+    #[test]
+    fn build_top_level_matcher_follow_config() {
+        let mut config = Config::default();
+
+        build_top_level_matcher(&["-follow"], &mut config).unwrap();
+
+        assert_eq!(config.follow, Follow::Always);
+    }
+
     #[test]
     fn comparable_value_matches() {
         assert!(
@@ -1477,4 +1633,54 @@ mod tests {
             }
         }
     }
+
+    #[test]
+    fn build_top_level_matcher_option_logical() {
+        let mut config = Config::default();
+        build_top_level_matcher(&["-maxdepth", "0", "-a", "-print"], &mut config)
+            .expect("logical operators like -a should work with options");
+        assert_eq!(config.max_depth, 0);
+    }
+
+    #[test]
+    fn build_top_level_matcher_help_invalid() {
+        let mut config = Config::default();
+        build_top_level_matcher(&["(", "-help", "-a"], &mut config)
+            .expect("-help should stop parsing");
+        assert!(config.help_requested);
+    }
+
+    #[test]
+    fn build_top_level_matcher_version_invalid() {
+        let mut config = Config::default();
+        build_top_level_matcher(&["(", "-version", "-o", ")", ")"], &mut config)
+            .expect("-version should stop parsing");
+        assert!(config.version_requested);
+    }
+
+    #[test]
+    fn get_or_create_file_test() {
+        use std::fs;
+
+        // remove file if hard link file exist.
+        // But you can't delete a file that doesn't exist,
+        // so ignore the error returned here.
+        let _ = fs::remove_file("test_data/get_or_create_file_test");
+
+        // test create file
+        let file = get_or_create_file("test_data/get_or_create_file_test");
+        assert!(file.is_ok());
+
+        let file = get_or_create_file("test_data/get_or_create_file_test");
+        assert!(file.is_ok());
+
+        // test error when file no permission
+        #[cfg(unix)]
+        {
+            let result = get_or_create_file("/etc/shadow");
+            assert!(result.is_err());
+        }
+
+        let _ = fs::remove_file("test_data/get_or_create_file_test");
+    }
 }
