@@ -7,21 +7,35 @@
 use std::error::Error;
 use std::fs::{self, Metadata};
 use std::io::{stderr, Write};
-use std::time::{SystemTime, UNIX_EPOCH};
-use walkdir::DirEntry;
+use std::time::{Duration, SystemTime, UNIX_EPOCH};
 
-use super::{ComparableValue, Matcher, MatcherIO};
+#[cfg(unix)]
+use std::os::unix::fs::MetadataExt;
+
+use super::{ComparableValue, Follow, Matcher, MatcherIO, WalkEntry};
 
 const SECONDS_PER_DAY: i64 = 60 * 60 * 24;
 
+fn get_time(matcher_io: &mut MatcherIO, today_start: bool) -> SystemTime {
+    if today_start {
+        // the time at 00:00:00 of today
+        let duration = matcher_io.now().duration_since(UNIX_EPOCH).unwrap();
+        let seconds = duration.as_secs();
+        let midnight_seconds = seconds - (seconds % 86400);
+        UNIX_EPOCH + Duration::from_secs(midnight_seconds)
+    } else {
+        matcher_io.now()
+    }
+}
+
 /// This matcher checks whether a file is newer than the file the matcher is initialized with.
 pub struct NewerMatcher {
     given_modification_time: SystemTime,
 }
 
 impl NewerMatcher {
-    pub fn new(path_to_file: &str) -> Result<Self, Box<dyn Error>> {
-        let metadata = fs::metadata(path_to_file)?;
+    pub fn new(path_to_file: &str, follow: Follow) -> Result<Self, Box<dyn Error>> {
+        let metadata = follow.root_metadata(path_to_file)?;
         Ok(Self {
             given_modification_time: metadata.modified()?,
         })
@@ -29,7 +43,7 @@ impl NewerMatcher {
 
     /// Implementation of matches that returns a result, allowing use to use try!
     /// to deal with the errors.
-    fn matches_impl(&self, file_info: &DirEntry) -> Result<bool, Box<dyn Error>> {
+    fn matches_impl(&self, file_info: &WalkEntry) -> Result<bool, Box<dyn Error>> {
         let this_time = file_info.metadata()?.modified()?;
         // duration_since returns an Ok duration if this_time <= given_modification_time
         // and returns an Err (with a duration) otherwise. So if this_time >
@@ -43,7 +57,7 @@ impl NewerMatcher {
 }
 
 impl Matcher for NewerMatcher {
-    fn matches(&self, file_info: &DirEntry, _: &mut MatcherIO) -> bool {
+    fn matches(&self, file_info: &WalkEntry, _: &mut MatcherIO) -> bool {
         match self.matches_impl(file_info) {
             Err(e) => {
                 writeln!(
@@ -75,6 +89,7 @@ pub enum NewerOptionType {
 }
 
 impl NewerOptionType {
+    #[allow(clippy::should_implement_trait)]
     pub fn from_str(option: &str) -> Self {
         match option {
             "a" => NewerOptionType::Accessed,
@@ -84,12 +99,11 @@ impl NewerOptionType {
         }
     }
 
-    fn get_file_time(self, metadata: Metadata) -> std::io::Result<SystemTime> {
+    fn get_file_time(self, metadata: &Metadata) -> std::io::Result<SystemTime> {
         match self {
             NewerOptionType::Accessed => metadata.accessed(),
             NewerOptionType::Birthed => metadata.created(),
-            // metadata.ctime() only impl in MetadataExt
-            NewerOptionType::Changed => metadata.accessed(),
+            NewerOptionType::Changed => metadata.changed(),
             NewerOptionType::Modified => metadata.modified(),
         }
     }
@@ -119,7 +133,7 @@ impl NewerOptionMatcher {
         })
     }
 
-    fn matches_impl(&self, file_info: &DirEntry) -> Result<bool, Box<dyn Error>> {
+    fn matches_impl(&self, file_info: &WalkEntry) -> Result<bool, Box<dyn Error>> {
         let x_option_time = self.x_option.get_file_time(file_info.metadata()?)?;
         let y_option_time = self.y_option.get_file_time(file_info.metadata()?)?;
 
@@ -135,7 +149,7 @@ impl NewerOptionMatcher {
 }
 
 impl Matcher for NewerOptionMatcher {
-    fn matches(&self, file_info: &DirEntry, _: &mut MatcherIO) -> bool {
+    fn matches(&self, file_info: &WalkEntry, _: &mut MatcherIO) -> bool {
         match self.matches_impl(file_info) {
             Err(e) => {
                 writeln!(
@@ -169,7 +183,7 @@ impl NewerTimeMatcher {
         }
     }
 
-    fn matches_impl(&self, file_info: &DirEntry) -> Result<bool, Box<dyn Error>> {
+    fn matches_impl(&self, file_info: &WalkEntry) -> Result<bool, Box<dyn Error>> {
         let this_time = self.newer_time_type.get_file_time(file_info.metadata()?)?;
         let timestamp = this_time
             .duration_since(UNIX_EPOCH)
@@ -186,7 +200,7 @@ impl NewerTimeMatcher {
 }
 
 impl Matcher for NewerTimeMatcher {
-    fn matches(&self, file_info: &DirEntry, _: &mut MatcherIO) -> bool {
+    fn matches(&self, file_info: &WalkEntry, _: &mut MatcherIO) -> bool {
         match self.matches_impl(file_info) {
             Err(e) => {
                 writeln!(
@@ -204,18 +218,47 @@ impl Matcher for NewerTimeMatcher {
     }
 }
 
+/// Provide access to the *change* timestamp, since std::fs::Metadata doesn't expose it.
+pub trait ChangeTime {
+    /// Returns the time of the last change to the metadata.
+    fn changed(&self) -> std::io::Result<SystemTime>;
+}
+
+#[cfg(unix)]
+impl ChangeTime for Metadata {
+    fn changed(&self) -> std::io::Result<SystemTime> {
+        let ctime_sec = self.ctime();
+        let ctime_nsec = self.ctime_nsec() as u32;
+        let ctime = if ctime_sec >= 0 {
+            UNIX_EPOCH + std::time::Duration::new(ctime_sec as u64, ctime_nsec)
+        } else {
+            UNIX_EPOCH - std::time::Duration::new(-ctime_sec as u64, ctime_nsec)
+        };
+        Ok(ctime)
+    }
+}
+
+#[cfg(not(unix))]
+impl ChangeTime for Metadata {
+    fn changed(&self) -> std::io::Result<SystemTime> {
+        // Rust's stdlib doesn't (yet) expose ChangeTime on Windows
+        // https://github.com/rust-lang/rust/issues/121478
+        Err(std::io::Error::from(std::io::ErrorKind::Unsupported))
+    }
+}
+
 #[derive(Clone, Copy, Debug)]
 pub enum FileTimeType {
     Accessed,
-    Created,
+    Changed,
     Modified,
 }
 
 impl FileTimeType {
-    fn get_file_time(self, metadata: Metadata) -> std::io::Result<SystemTime> {
+    fn get_file_time(self, metadata: &Metadata) -> std::io::Result<SystemTime> {
         match self {
             FileTimeType::Accessed => metadata.accessed(),
-            FileTimeType::Created => metadata.created(),
+            FileTimeType::Changed => metadata.changed(),
             FileTimeType::Modified => metadata.modified(),
         }
     }
@@ -226,11 +269,13 @@ impl FileTimeType {
 pub struct FileTimeMatcher {
     days: ComparableValue,
     file_time_type: FileTimeType,
+    today_start: bool,
 }
 
 impl Matcher for FileTimeMatcher {
-    fn matches(&self, file_info: &DirEntry, matcher_io: &mut MatcherIO) -> bool {
-        match self.matches_impl(file_info, matcher_io.now()) {
+    fn matches(&self, file_info: &WalkEntry, matcher_io: &mut MatcherIO) -> bool {
+        let start_time = get_time(matcher_io, self.today_start);
+        match self.matches_impl(file_info, start_time) {
             Err(e) => {
                 writeln!(
                     &mut stderr(),
@@ -250,12 +295,16 @@ impl Matcher for FileTimeMatcher {
 impl FileTimeMatcher {
     /// Implementation of matches that returns a result, allowing use to use try!
     /// to deal with the errors.
-    fn matches_impl(&self, file_info: &DirEntry, now: SystemTime) -> Result<bool, Box<dyn Error>> {
+    fn matches_impl(
+        &self,
+        file_info: &WalkEntry,
+        start_time: SystemTime,
+    ) -> Result<bool, Box<dyn Error>> {
         let this_time = self.file_time_type.get_file_time(file_info.metadata()?)?;
         let mut is_negative = false;
         // durations can't be negative. So duration_since returns a duration
         // wrapped in an error if now < this_time.
-        let age = match now.duration_since(this_time) {
+        let age = match start_time.duration_since(this_time) {
             Ok(duration) => duration,
             Err(e) => {
                 is_negative = true;
@@ -263,19 +312,29 @@ impl FileTimeMatcher {
             }
         };
         let age_in_seconds: i64 = age.as_secs() as i64 * if is_negative { -1 } else { 1 };
+
         // rust division truncates towards zero (see
         // https://github.com/rust-lang/rust/blob/master/src/libcore/ops.rs#L580 )
         // so a simple age_in_seconds / SECONDS_PER_DAY gives the wrong answer
         // for negative ages: a file whose age is 1 second in the future needs to
         // count as -1 day old, not 0.
-        let age_in_days = age_in_seconds / SECONDS_PER_DAY + if is_negative { -1 } else { 0 };
+        // If today_start is true, we should count it as 0 days old.
+        // because today is 00:00:00, so we need to subtract 1 day.
+        let negative_offset = if is_negative && !self.today_start {
+            -1
+        } else {
+            0
+        };
+
+        let age_in_days = age_in_seconds / SECONDS_PER_DAY + negative_offset;
         Ok(self.days.imatches(age_in_days))
     }
 
-    pub fn new(file_time_type: FileTimeType, days: ComparableValue) -> Self {
+    pub fn new(file_time_type: FileTimeType, days: ComparableValue, today_start: bool) -> Self {
         Self {
             days,
             file_time_type,
+            today_start,
         }
     }
 }
@@ -283,11 +342,13 @@ impl FileTimeMatcher {
 pub struct FileAgeRangeMatcher {
     minutes: ComparableValue,
     file_time_type: FileTimeType,
+    today_start: bool,
 }
 
 impl Matcher for FileAgeRangeMatcher {
-    fn matches(&self, file_info: &DirEntry, matcher_io: &mut MatcherIO) -> bool {
-        match self.matches_impl(file_info, matcher_io.now()) {
+    fn matches(&self, file_info: &WalkEntry, matcher_io: &mut MatcherIO) -> bool {
+        let start_time = get_time(matcher_io, self.today_start);
+        match self.matches_impl(file_info, start_time) {
             Err(e) => {
                 writeln!(
                     &mut stderr(),
@@ -305,10 +366,14 @@ impl Matcher for FileAgeRangeMatcher {
 }
 
 impl FileAgeRangeMatcher {
-    fn matches_impl(&self, file_info: &DirEntry, now: SystemTime) -> Result<bool, Box<dyn Error>> {
+    fn matches_impl(
+        &self,
+        file_info: &WalkEntry,
+        start_time: SystemTime,
+    ) -> Result<bool, Box<dyn Error>> {
         let this_time = self.file_time_type.get_file_time(file_info.metadata()?)?;
         let mut is_negative = false;
-        let age = match now.duration_since(this_time) {
+        let age = match start_time.duration_since(this_time) {
             Ok(duration) => duration,
             Err(e) => {
                 is_negative = true;
@@ -320,10 +385,11 @@ impl FileAgeRangeMatcher {
         Ok(self.minutes.imatches(age_in_minutes))
     }
 
-    pub fn new(file_time_type: FileTimeType, minutes: ComparableValue) -> Self {
+    pub fn new(file_time_type: FileTimeType, minutes: ComparableValue, today_start: bool) -> Self {
         Self {
             minutes,
             file_time_type,
+            today_start,
         }
     }
 }
@@ -354,9 +420,13 @@ mod tests {
 
         let new_file = get_dir_entry_for(&temp_dir_path, new_file_name);
 
-        let matcher_for_new =
-            NewerMatcher::new(&temp_dir.path().join(new_file_name).to_string_lossy()).unwrap();
-        let matcher_for_old = NewerMatcher::new(&old_file.path().to_string_lossy()).unwrap();
+        let matcher_for_new = NewerMatcher::new(
+            &temp_dir.path().join(new_file_name).to_string_lossy(),
+            Follow::Never,
+        )
+        .unwrap();
+        let matcher_for_old =
+            NewerMatcher::new(&old_file.path().to_string_lossy(), Follow::Never).unwrap();
         let deps = FakeDependencies::new();
 
         assert!(
@@ -381,13 +451,13 @@ mod tests {
         let files_mtime = file.metadata().unwrap().modified().unwrap();
 
         let exactly_one_day_matcher =
-            FileTimeMatcher::new(FileTimeType::Modified, ComparableValue::EqualTo(1));
+            FileTimeMatcher::new(FileTimeType::Modified, ComparableValue::EqualTo(1), false);
         let more_than_one_day_matcher =
-            FileTimeMatcher::new(FileTimeType::Modified, ComparableValue::MoreThan(1));
+            FileTimeMatcher::new(FileTimeType::Modified, ComparableValue::MoreThan(1), false);
         let less_than_one_day_matcher =
-            FileTimeMatcher::new(FileTimeType::Modified, ComparableValue::LessThan(1));
+            FileTimeMatcher::new(FileTimeType::Modified, ComparableValue::LessThan(1), false);
         let zero_day_matcher =
-            FileTimeMatcher::new(FileTimeType::Modified, ComparableValue::EqualTo(0));
+            FileTimeMatcher::new(FileTimeType::Modified, ComparableValue::EqualTo(0), false);
 
         // set "now" to 2 days after the file was modified.
         let mut deps = FakeDependencies::new();
@@ -469,9 +539,87 @@ mod tests {
     }
 
     #[test]
-    fn file_time_matcher_modified_created_accessed() {
+    fn file_time_matcher_with_daystart() {
+        // this file should already exist
+        let file = get_dir_entry_for("test_data", "simple");
+
+        let mut deps = FakeDependencies::new();
+        let files_mtime = file.metadata().unwrap().modified().unwrap();
+
+        let exactly_one_day_matcher =
+            FileTimeMatcher::new(FileTimeType::Modified, ComparableValue::EqualTo(1), true);
+        let more_than_one_day_matcher =
+            FileTimeMatcher::new(FileTimeType::Modified, ComparableValue::MoreThan(1), true);
+        let less_than_one_day_matcher =
+            FileTimeMatcher::new(FileTimeType::Modified, ComparableValue::LessThan(1), true);
+        let zero_day_matcher =
+            FileTimeMatcher::new(FileTimeType::Modified, ComparableValue::EqualTo(0), true);
+
+        // set "now" to 3 days after the file was modified.
+        // Because daystart affects the time when the calculation starts,
+        // in order to avoid complicated assertions, it is set to 3 days later.
+        deps.set_time(files_mtime + Duration::new(3 * SECONDS_PER_DAY as u64, 0));
+        assert!(
+            !exactly_one_day_matcher.matches(&file, &mut deps.new_matcher_io()),
+            "3 day old file shouldn't match exactly 1 day old"
+        );
+        assert!(
+            more_than_one_day_matcher.matches(&file, &mut deps.new_matcher_io()),
+            "3 day old file should match more than 1 day old"
+        );
+        assert!(
+            !less_than_one_day_matcher.matches(&file, &mut deps.new_matcher_io()),
+            "3 day old file shouldn't match less than 1 day old"
+        );
+        assert!(
+            !zero_day_matcher.matches(&file, &mut deps.new_matcher_io()),
+            "3 day old file shouldn't match exactly 0 days old"
+        );
+
+        // set "now" to exactly the same time file was modified.
+        deps.set_time(files_mtime);
+        assert!(
+            !exactly_one_day_matcher.matches(&file, &mut deps.new_matcher_io()),
+            "0 day old file shouldn't match exactly 1 day old"
+        );
+        assert!(
+            !more_than_one_day_matcher.matches(&file, &mut deps.new_matcher_io()),
+            "0 day old file shouldn't match more than 1 day old"
+        );
+        assert!(
+            less_than_one_day_matcher.matches(&file, &mut deps.new_matcher_io()),
+            "0 day old file should match less than 1 day old"
+        );
+        assert!(
+            zero_day_matcher.matches(&file, &mut deps.new_matcher_io()),
+            "0 day old file should match exactly 0 days old"
+        );
+
+        // set "now" to a second before the file was modified (e.g. the file was
+        // modified after find started running
+        deps.set_time(files_mtime - Duration::new(1_u64, 0));
+        assert!(
+            !exactly_one_day_matcher.matches(&file, &mut deps.new_matcher_io()),
+            "future-modified file shouldn't match exactly 1 day old"
+        );
+        assert!(
+            !more_than_one_day_matcher.matches(&file, &mut deps.new_matcher_io()),
+            "future-modified file shouldn't match more than 1 day old"
+        );
+        assert!(
+            less_than_one_day_matcher.matches(&file, &mut deps.new_matcher_io()),
+            "future-modified file should match less than 1 day old"
+        );
+        assert!(
+            zero_day_matcher.matches(&file, &mut deps.new_matcher_io()),
+            "future-modified file should match exactly 0 days old"
+        );
+    }
+
+    #[test]
+    fn file_time_matcher_modified_changed_accessed() {
         let temp_dir = Builder::new()
-            .prefix("file_time_matcher_modified_created_accessed")
+            .prefix("file_time_matcher_modified_changed_accessed")
             .tempdir()
             .unwrap();
 
@@ -521,8 +669,8 @@ mod tests {
             test_matcher_for_file_time_type(&file_info, accessed_time, FileTimeType::Accessed);
         }
 
-        if let Ok(creation_time) = metadata.created() {
-            test_matcher_for_file_time_type(&file_info, creation_time, FileTimeType::Created);
+        if let Ok(creation_time) = metadata.changed() {
+            test_matcher_for_file_time_type(&file_info, creation_time, FileTimeType::Changed);
         }
 
         if let Ok(modified_time) = metadata.modified() {
@@ -530,14 +678,14 @@ mod tests {
         }
     }
 
-    /// helper function for `file_time_matcher_modified_created_accessed`
+    /// helper function for `file_time_matcher_modified_changed_accessed`
     fn test_matcher_for_file_time_type(
-        file_info: &DirEntry,
+        file_info: &WalkEntry,
         file_time: SystemTime,
         file_time_type: FileTimeType,
     ) {
         {
-            let matcher = FileTimeMatcher::new(file_time_type, ComparableValue::EqualTo(0));
+            let matcher = FileTimeMatcher::new(file_time_type, ComparableValue::EqualTo(0), false);
 
             let mut deps = FakeDependencies::new();
             deps.set_time(file_time);
@@ -556,10 +704,14 @@ mod tests {
 
     #[test]
     fn newer_option_matcher() {
-        #[cfg(target_os = "linux")]
-        let options = ["a", "c", "m"];
-        #[cfg(not(target_os = "linux"))]
-        let options = ["a", "B", "c", "m"];
+        let options = [
+            "a",
+            #[cfg(not(target_os = "linux"))]
+            "B",
+            #[cfg(unix)]
+            "c",
+            "m",
+        ];
 
         for x_option in options {
             for y_option in options {
@@ -584,30 +736,6 @@ mod tests {
                         .matches(&new_file, &mut deps.new_matcher_io()),
                     "new_file should be newer than old_dir"
                 );
-
-                // After the file is deleted, DirEntry will point to an empty file location,
-                // thus causing the Matcher to generate an IO error after matching.
-                //
-                // Note: This test is nondeterministic on Windows,
-                // because fs::remove_file may not actually remove the file from
-                // the file system even if it returns Ok.
-                // Therefore, this test will only be performed on Linux/Unix.
-                let _ = fs::remove_file(&*new_file.path().to_string_lossy());
-
-                #[cfg(unix)]
-                {
-                    let matcher = NewerOptionMatcher::new(
-                        x_option.to_string(),
-                        y_option.to_string(),
-                        &old_file.path().to_string_lossy(),
-                    );
-                    assert!(
-                        !matcher
-                            .unwrap()
-                            .matches(&new_file, &mut deps.new_matcher_io()),
-                        "The correct situation is that the file reading here cannot be successful."
-                    );
-                }
             }
         }
     }
@@ -669,45 +797,24 @@ mod tests {
             "file modified time should after 'time'"
         );
 
-        let inode_changed_matcher = NewerTimeMatcher::new(NewerOptionType::Changed, time);
-        // Steps to change inode:
-        // 1. Copy and rename the file
-        // 2. Delete the old file
-        // 3. Change the new file name to the old file name
-        let _ = File::create(temp_dir.path().join("inode_test_file")).expect("create temp file");
-        let _ = fs::copy("inode_test_file", "new_inode_test_file");
-        let _ = fs::remove_file("inode_test_file");
-        let _ = fs::rename("new_inode_test_file", "inode_test_file");
-        let file_info = get_dir_entry_for(&temp_dir.path().to_string_lossy(), "inode_test_file");
-        assert!(
-            inode_changed_matcher.matches(&file_info, &mut deps.new_matcher_io()),
-            "file inode changed time should after 'std_time'"
-        );
-
-        // After the file is deleted, DirEntry will point to an empty file location,
-        // thus causing the Matcher to generate an IO error after matching.
-        //
-        // Note: This test is nondeterministic on Windows,
-        // because fs::remove_file may not actually remove the file from
-        // the file system even if it returns Ok.
-        // Therefore, this test will only be performed on Linux/Unix.
-        let _ = fs::remove_file(&*file_info.path().to_string_lossy());
-
         #[cfg(unix)]
         {
-            let matchers = [
-                &created_matcher,
-                &accessed_matcher,
-                &modified_matcher,
-                &inode_changed_matcher,
-            ];
-
-            for matcher in &matchers {
-                assert!(
-                    !matcher.matches(&file_info, &mut deps.new_matcher_io()),
-                    "The correct situation is that the file reading here cannot be successful."
-                );
-            }
+            let inode_changed_matcher = NewerTimeMatcher::new(NewerOptionType::Changed, time);
+            // Steps to change inode:
+            // 1. Copy and rename the file
+            // 2. Delete the old file
+            // 3. Change the new file name to the old file name
+            let _ =
+                File::create(temp_dir.path().join("inode_test_file")).expect("create temp file");
+            let _ = fs::copy("inode_test_file", "new_inode_test_file");
+            let _ = fs::remove_file("inode_test_file");
+            let _ = fs::rename("new_inode_test_file", "inode_test_file");
+            let file_info =
+                get_dir_entry_for(&temp_dir.path().to_string_lossy(), "inode_test_file");
+            assert!(
+                inode_changed_matcher.matches(&file_info, &mut deps.new_matcher_io()),
+                "file inode changed time should after 'std_time'"
+            );
         }
     }
 
@@ -728,20 +835,21 @@ mod tests {
         // Means to find files accessed / modified more than 1 minute ago.
         [
             FileTimeType::Accessed,
-            FileTimeType::Created,
+            FileTimeType::Changed,
             FileTimeType::Modified,
         ]
         .iter()
         .for_each(|time_type| {
-            let more_matcher = FileAgeRangeMatcher::new(*time_type, ComparableValue::MoreThan(1));
+            let more_matcher =
+                FileAgeRangeMatcher::new(*time_type, ComparableValue::MoreThan(1), true);
             assert!(
                 !more_matcher.matches(&new_file, &mut FakeDependencies::new().new_matcher_io()),
                 "{}",
                 format!(
-                    "more minutes old file should match more than 1 minute old in {} test.",
+                    "more minutes old file should not match more than 1 minute old in {} test.",
                     match *time_type {
                         FileTimeType::Accessed => "accessed",
-                        FileTimeType::Created => "created",
+                        FileTimeType::Changed => "changed",
                         FileTimeType::Modified => "modified",
                     }
                 )
@@ -756,20 +864,22 @@ mod tests {
         // Means to find files accessed / modified less than 1 minute ago.
         [
             FileTimeType::Accessed,
-            FileTimeType::Created,
+            #[cfg(unix)]
+            FileTimeType::Changed,
             FileTimeType::Modified,
         ]
         .iter()
         .for_each(|time_type| {
-            let less_matcher = FileAgeRangeMatcher::new(*time_type, ComparableValue::LessThan(1));
+            let less_matcher =
+                FileAgeRangeMatcher::new(*time_type, ComparableValue::LessThan(1), true);
             assert!(
                 less_matcher.matches(&new_file, &mut FakeDependencies::new().new_matcher_io()),
                 "{}",
                 format!(
-                    "less minutes old file should not match less than 1 minute old in {} test.",
+                    "less minutes old file should match less than 1 minute old in {} test.",
                     match *time_type {
                         FileTimeType::Accessed => "accessed",
-                        FileTimeType::Created => "created",
+                        FileTimeType::Changed => "changed",
                         FileTimeType::Modified => "modified",
                     }
                 )
@@ -779,7 +889,7 @@ mod tests {
         // catch file error
         let _ = fs::remove_file(&*new_file.path().to_string_lossy());
         let matcher =
-            FileAgeRangeMatcher::new(FileTimeType::Modified, ComparableValue::MoreThan(1));
+            FileAgeRangeMatcher::new(FileTimeType::Modified, ComparableValue::MoreThan(1), true);
         assert!(
             !matcher.matches(&new_file, &mut FakeDependencies::new().new_matcher_io()),
             "The correct situation is that the file reading here cannot be successful."
