@@ -6,6 +6,7 @@
 
 pub mod matchers;
 
+use matchers::{Follow, WalkEntry};
 use std::cell::RefCell;
 use std::error::Error;
 use std::io::{stderr, stdout, Write};
@@ -21,6 +22,9 @@ pub struct Config {
     sorted_output: bool,
     help_requested: bool,
     version_requested: bool,
+    today_start: bool,
+    no_leaf_dirs: bool,
+    follow: Follow,
 }
 
 impl Default for Config {
@@ -33,15 +37,21 @@ impl Default for Config {
             sorted_output: false,
             help_requested: false,
             version_requested: false,
+            today_start: false,
+            // Directory information and traversal are done by walkdir,
+            // and this configuration field will exist as
+            // a compatibility item for GNU findutils.
+            no_leaf_dirs: false,
+            follow: Follow::Never,
         }
     }
 }
 
 /// Trait that encapsulates various dependencies (output, clocks, etc.) that we
 /// might want to fake out for unit tests.
-pub trait Dependencies<'a> {
-    fn get_output(&'a self) -> &'a RefCell<dyn Write>;
-    fn now(&'a self) -> SystemTime;
+pub trait Dependencies {
+    fn get_output(&self) -> &RefCell<dyn Write>;
+    fn now(&self) -> SystemTime;
 }
 
 /// Struct that holds the dependencies we use when run as the real executable.
@@ -66,12 +76,12 @@ impl Default for StandardDependencies {
     }
 }
 
-impl<'a> Dependencies<'a> for StandardDependencies {
-    fn get_output(&'a self) -> &'a RefCell<dyn Write> {
+impl Dependencies for StandardDependencies {
+    fn get_output(&self) -> &RefCell<dyn Write> {
         self.output.as_ref()
     }
 
-    fn now(&'a self) -> SystemTime {
+    fn now(&self) -> SystemTime {
         self.now
     }
 }
@@ -94,9 +104,9 @@ fn parse_args(args: &[&str]) -> Result<ParsedInfo, Box<dyn Error>> {
             "-O0" | "-O1" | "-O2" | "-O3" => {
                 // GNU find optimization level flag (ignored)
             }
-            "-P" => {
-                // Never follow symlinks (the default)
-            }
+            "-H" => config.follow = Follow::Roots,
+            "-L" => config.follow = Follow::Always,
+            "-P" => config.follow = Follow::Never,
             "--" => {
                 // End of flags
                 i += 1;
@@ -128,36 +138,42 @@ fn parse_args(args: &[&str]) -> Result<ParsedInfo, Box<dyn Error>> {
     })
 }
 
-fn process_dir<'a>(
+fn process_dir(
     dir: &str,
     config: &Config,
-    deps: &'a dyn Dependencies<'a>,
+    deps: &dyn Dependencies,
     matcher: &dyn matchers::Matcher,
     quit: &mut bool,
-) -> u64 {
-    let mut found_count: u64 = 0;
+) -> i32 {
     let mut walkdir = WalkDir::new(dir)
         .contents_first(config.depth_first)
         .max_depth(config.max_depth)
         .min_depth(config.min_depth)
-        .same_file_system(config.same_file_system);
+        .same_file_system(config.same_file_system)
+        .follow_links(config.follow == Follow::Always)
+        .follow_root_links(config.follow != Follow::Never);
     if config.sorted_output {
         walkdir = walkdir.sort_by(|a, b| a.file_name().cmp(b.file_name()));
     }
 
+    let mut ret = 0;
+
     // Slightly yucky loop handling here :-(. See docs for
     // WalkDirIterator::skip_current_dir for explanation.
     let mut it = walkdir.into_iter();
     while let Some(result) = it.next() {
-        match result {
+        match WalkEntry::from_walkdir(result, config.follow) {
             Err(err) => {
-                uucore::error::set_exit_code(1);
-                writeln!(&mut stderr(), "Error: {dir}: {err}").unwrap()
+                ret = 1;
+                writeln!(&mut stderr(), "Error: {err}").unwrap()
             }
             Ok(entry) => {
                 let mut matcher_io = matchers::MatcherIO::new(deps);
-                if matcher.matches(&entry, &mut matcher_io) {
-                    found_count += 1;
+
+                matcher.matches(&entry, &mut matcher_io);
+                match matcher_io.exit_code() {
+                    0 => {}
+                    code => ret = code,
                 }
                 if matcher_io.should_quit() {
                     *quit = true;
@@ -169,10 +185,11 @@ fn process_dir<'a>(
             }
         }
     }
-    found_count
+
+    ret
 }
 
-fn do_find<'a>(args: &[&str], deps: &'a dyn Dependencies<'a>) -> Result<u64, Box<dyn Error>> {
+fn do_find(args: &[&str], deps: &dyn Dependencies) -> Result<i32, Box<dyn Error>> {
     let paths_and_matcher = parse_args(args)?;
     if paths_and_matcher.config.help_requested {
         print_help();
@@ -183,21 +200,25 @@ fn do_find<'a>(args: &[&str], deps: &'a dyn Dependencies<'a>) -> Result<u64, Box
         return Ok(0);
     }
 
-    let mut found_count: u64 = 0;
+    let mut ret = 0;
     let mut quit = false;
     for path in paths_and_matcher.paths {
-        found_count += process_dir(
+        let dir_ret = process_dir(
             &path,
             &paths_and_matcher.config,
             deps,
             &*paths_and_matcher.matcher,
             &mut quit,
         );
+        if dir_ret != 0 {
+            ret = dir_ret;
+        }
         if quit {
             break;
         }
     }
-    Ok(found_count)
+
+    Ok(ret)
 }
 
 fn print_help() {
@@ -255,9 +276,9 @@ fn print_version() {
 /// All main has to do is pass in the command-line args and exit the process
 /// with the exit code. Note that the first string in args is expected to be
 /// the name of the executable.
-pub fn find_main<'a>(args: &[&str], deps: &'a dyn Dependencies<'a>) -> i32 {
+pub fn find_main(args: &[&str], deps: &dyn Dependencies) -> i32 {
     match do_find(&args[1..], deps) {
-        Ok(_) => uucore::error::get_exit_code(),
+        Ok(ret) => ret,
         Err(e) => {
             writeln!(&mut stderr(), "Error: {e}").unwrap();
             1
@@ -279,6 +300,7 @@ mod tests {
     #[cfg(windows)]
     use std::os::windows::fs::symlink_file;
 
+    use crate::find::matchers::time::ChangeTime;
     use crate::find::matchers::MatcherIO;
 
     use super::*;
@@ -327,12 +349,12 @@ mod tests {
         }
     }
 
-    impl<'a> Dependencies<'a> for FakeDependencies {
-        fn get_output(&'a self) -> &'a RefCell<dyn Write> {
+    impl Dependencies for FakeDependencies {
+        fn get_output(&self) -> &RefCell<dyn Write> {
             &self.output
         }
 
-        fn now(&'a self) -> SystemTime {
+        fn now(&self) -> SystemTime {
             self.now
         }
     }
@@ -380,9 +402,22 @@ mod tests {
         assert_eq!(parsed_info.paths, ["."]);
     }
 
+    #[test]
+    fn parse_h_flag() {
+        let parsed_info = super::parse_args(&["-H"]).expect("parsing should succeed");
+        assert_eq!(parsed_info.config.follow, Follow::Roots);
+    }
+
+    #[test]
+    fn parse_l_flag() {
+        let parsed_info = super::parse_args(&["-L"]).expect("parsing should succeed");
+        assert_eq!(parsed_info.config.follow, Follow::Always);
+    }
+
     #[test]
     fn parse_p_flag() {
-        super::parse_args(&["-P"]).expect("parsing should succeed");
+        let parsed_info = super::parse_args(&["-P"]).expect("parsing should succeed");
+        assert_eq!(parsed_info.config.follow, Follow::Never);
     }
 
     #[test]
@@ -667,7 +702,7 @@ mod tests {
         let meta = fs::metadata("./test_data/simple/subdir/ABBBC").unwrap();
 
         // metadata can return errors like StringError("creation time is not available on this platform currently")
-        // so skip tests that won't pass due to shortcomings in std:;fs.
+        // so skip tests that won't pass due to shortcomings in std::fs.
         if let Ok(file_time) = meta.modified() {
             file_time_helper(file_time, "-mtime");
         }
@@ -678,8 +713,8 @@ mod tests {
         let meta = fs::metadata("./test_data/simple/subdir/ABBBC").unwrap();
 
         // metadata can return errors like StringError("creation time is not available on this platform currently")
-        // so skip tests that won't pass due to shortcomings in std:;fs.
-        if let Ok(file_time) = meta.created() {
+        // so skip tests that won't pass due to shortcomings in std::fs.
+        if let Ok(file_time) = meta.changed() {
             file_time_helper(file_time, "-ctime");
         }
     }
@@ -689,7 +724,7 @@ mod tests {
         let meta = fs::metadata("./test_data/simple/subdir/ABBBC").unwrap();
 
         // metadata can return errors like StringError("creation time is not available on this platform currently")
-        // so skip tests that won't pass due to shortcomings in std:;fs.
+        // so skip tests that won't pass due to shortcomings in std::fs.
         if let Ok(file_time) = meta.accessed() {
             file_time_helper(file_time, "-atime");
         }
@@ -912,6 +947,20 @@ mod tests {
                 );
 
                 assert_eq!(rc, 0);
+
+                let arg = &format!("-follow -newer{x}{y}").to_string();
+                let deps = FakeDependencies::new();
+                let rc = find_main(
+                    &[
+                        "find",
+                        "./test_data/simple/subdir",
+                        arg,
+                        "./test_data/simple/subdir/ABBBC",
+                    ],
+                    &deps,
+                );
+
+                assert_eq!(rc, 0);
             }
         }
     }
@@ -937,9 +986,10 @@ mod tests {
         }
     }
 
+    #[cfg(unix)]
     #[test]
-    fn test_find_newer_xy_before_created_time() {
-        // normal - before the created time
+    fn test_find_newer_xy_before_changed_time() {
+        // normal - before the changed time
         #[cfg(target_os = "linux")]
         let args = ["-newerat", "-newerct", "-newermt"];
         #[cfg(not(target_os = "linux"))]
@@ -961,8 +1011,8 @@ mod tests {
     }
 
     #[test]
-    fn test_find_newer_xy_after_created_time() {
-        // normal - after the created time
+    fn test_find_newer_xy_after_changed_time() {
+        // normal - after the changed time
         #[cfg(target_os = "linux")]
         let args = ["-newerat", "-newerct", "-newermt"];
         #[cfg(not(target_os = "linux"))]
@@ -1040,10 +1090,6 @@ mod tests {
 
         assert_eq!(rc, 1);
 
-        // Reset the exit code global variable in case we run another test after this one
-        // See https://github.com/uutils/coreutils/issues/5777
-        uucore::error::set_exit_code(0);
-
         if path.exists() {
             let _result = fs::create_dir(path);
             // Remove the unreadable and writable status of the file to avoid affecting other tests.
@@ -1200,10 +1246,12 @@ mod tests {
     fn test_fs_matcher() {
         use crate::find::tests::FakeDependencies;
         use matchers::fs::get_file_system_type;
+        use std::cell::RefCell;
         use std::path::Path;
 
         let path = Path::new("./test_data/simple/subdir");
-        let target_fs_type = get_file_system_type(path).unwrap();
+        let empty_cache = RefCell::new(None);
+        let target_fs_type = get_file_system_type(path, &empty_cache).unwrap();
 
         // should match fs type
         let deps = FakeDependencies::new();
@@ -1219,4 +1267,175 @@ mod tests {
 
         assert_eq!(rc, 0);
     }
+
+    #[test]
+    #[cfg(unix)]
+    fn test_noleaf() {
+        use crate::find::tests::FakeDependencies;
+
+        let deps = FakeDependencies::new();
+        let rc = find_main(&["find", "./test_data/simple/subdir", "-noleaf"], &deps);
+
+        assert_eq!(rc, 0);
+    }
+
+    #[test]
+    fn find_maxdepth_and() {
+        let deps = FakeDependencies::new();
+        let rc = find_main(
+            &[
+                "find",
+                &fix_up_slashes("./test_data/depth"),
+                "-maxdepth",
+                "0",
+                "-a",
+                "-print",
+            ],
+            &deps,
+        );
+
+        assert_eq!(rc, 0);
+        assert_eq!(
+            deps.get_output_as_string(),
+            fix_up_slashes("./test_data/depth\n")
+        );
+    }
+
+    #[test]
+    #[cfg(unix)]
+    fn test_daystart() {
+        use crate::find::tests::FakeDependencies;
+
+        let deps = FakeDependencies::new();
+        let rc = find_main(
+            &[
+                "find",
+                "./test_data/simple/subdir",
+                "-daystart",
+                "-mtime",
+                "0",
+            ],
+            &deps,
+        );
+
+        assert_eq!(rc, 0);
+
+        // twice -daystart should be matched
+        let deps = FakeDependencies::new();
+        let rc = find_main(
+            &[
+                "find",
+                "./test_data/simple/subdir",
+                "-daystart",
+                "-daystart",
+                "-mtime",
+                "1",
+            ],
+            &deps,
+        );
+
+        assert_eq!(rc, 0);
+    }
+
+    #[test]
+    fn find_fprint() {
+        let deps = FakeDependencies::new();
+        let rc = find_main(
+            &[
+                "find",
+                "./test_data/simple",
+                "-fprint",
+                "test_data/find_fprint",
+            ],
+            &deps,
+        );
+        assert_eq!(rc, 0);
+
+        let _ = fs::remove_file("test_data/find_fprint");
+    }
+
+    #[test]
+    fn test_follow() {
+        let deps = FakeDependencies::new();
+        let rc = find_main(&["find", "./test_data/simple", "-follow"], &deps);
+        assert_eq!(rc, 0);
+    }
+
+    #[cfg(unix)]
+    #[test]
+    fn test_h_flag() {
+        let deps = FakeDependencies::new();
+
+        let rc = find_main(
+            &["find", "-H", &fix_up_slashes("./test_data/links/link-d")],
+            &deps,
+        );
+
+        assert_eq!(rc, 0);
+        assert_eq!(
+            deps.get_output_as_string(),
+            fix_up_slashes(
+                "./test_data/links/link-d\n\
+                 ./test_data/links/link-d/test\n"
+            )
+        );
+    }
+
+    #[cfg(unix)]
+    #[test]
+    fn test_l_flag() {
+        let deps = FakeDependencies::new();
+
+        let rc = find_main(
+            &[
+                "find",
+                "-L",
+                &fix_up_slashes("./test_data/links"),
+                "-sorted",
+            ],
+            &deps,
+        );
+
+        assert_eq!(rc, 1);
+        assert_eq!(
+            deps.get_output_as_string(),
+            fix_up_slashes(
+                "./test_data/links\n\
+                 ./test_data/links/abbbc\n\
+                 ./test_data/links/link-d\n\
+                 ./test_data/links/link-d/test\n\
+                 ./test_data/links/link-f\n\
+                 ./test_data/links/link-missing\n\
+                 ./test_data/links/link-notdir\n\
+                 ./test_data/links/subdir\n\
+                 ./test_data/links/subdir/test\n"
+            )
+        );
+    }
+
+    #[cfg(unix)]
+    #[test]
+    fn test_p_flag() {
+        let deps = FakeDependencies::new();
+
+        let rc = find_main(
+            &["find", "-P", &fix_up_slashes("./test_data/links/link-d")],
+            &deps,
+        );
+
+        assert_eq!(rc, 0);
+        assert_eq!(
+            deps.get_output_as_string(),
+            fix_up_slashes("./test_data/links/link-d\n")
+        );
+    }
+
+    #[test]
+    #[cfg(unix)]
+    fn test_ls() {
+        let deps = FakeDependencies::new();
+        let rc = find_main(&["find", "./test_data/simple/subdir", "-ls"], &deps);
+
+        assert_eq!(rc, 0);
+    }
 }
