@@ -14,7 +14,7 @@ use std::{
     process::{Command, Stdio},
 };
 
-use clap::{crate_version, App, AppSettings, Arg};
+use clap::{crate_version, Arg, ArgAction};
 
 mod options {
     pub const COMMAND: &str = "COMMAND";
@@ -23,11 +23,13 @@ mod options {
     pub const DELIMITER: &str = "delimiter";
     pub const EXIT: &str = "exit";
     pub const MAX_ARGS: &str = "max-args";
+    pub const MAX_CHARS: &str = "max-chars";
     pub const MAX_LINES: &str = "max-lines";
     pub const MAX_PROCS: &str = "max-procs";
     pub const NO_RUN_IF_EMPTY: &str = "no-run-if-empty";
     pub const NULL: &str = "null";
-    pub const SIZE: &str = "size";
+    pub const REPLACE: &str = "replace";
+    pub const REPLACE_I: &str = "replace-I";
     pub const VERBOSE: &str = "verbose";
 }
 
@@ -36,10 +38,11 @@ struct Options {
     delimiter: Option<u8>,
     exit_if_pass_char_limit: bool,
     max_args: Option<usize>,
+    max_chars: Option<usize>,
     max_lines: Option<usize>,
     no_run_if_empty: bool,
     null: bool,
-    size: Option<usize>,
+    replace: Option<String>,
     verbose: bool,
 }
 
@@ -76,7 +79,7 @@ trait CommandSizeLimiter {
 }
 
 /// A pointer to the next limiter. A limiter should *always* call the cursor's
-/// try_next *before* updating its own state, to ensure that all other limiters
+/// `try_next` *before* updating its own state, to ensure that all other limiters
 /// are okay with the argument first.
 struct LimiterCursor<'collection> {
     limiters: &'collection mut [Box<dyn CommandSizeLimiter>],
@@ -303,6 +306,7 @@ impl CommandResult {
     }
 }
 
+#[allow(dead_code)] // `Killed` variant is never constructed on Windows
 #[derive(Debug)]
 enum CommandExecutionError {
     // exit code 255
@@ -318,9 +322,9 @@ impl Display for CommandExecutionError {
         match self {
             CommandExecutionError::UrgentlyFailed => write!(f, "Command exited with code 255"),
             CommandExecutionError::Killed { signal } => {
-                write!(f, "Command was killed with signal {}", signal)
+                write!(f, "Command was killed with signal {signal}")
             }
-            CommandExecutionError::CannotRun(err) => write!(f, "Command could not be run: {}", err),
+            CommandExecutionError::CannotRun(err) => write!(f, "Command could not be run: {err}"),
             CommandExecutionError::NotFound => write!(f, "Command not found"),
             CommandExecutionError::Unknown => write!(f, "Unknown error running command"),
         }
@@ -340,16 +344,17 @@ struct CommandBuilderOptions {
     limiters: LimiterCollection,
     verbose: bool,
     close_stdin: bool,
+    replace: Option<String>,
 }
-
 impl CommandBuilderOptions {
     fn new(
         action: ExecAction,
         env: HashMap<OsString, OsString>,
         mut limiters: LimiterCollection,
+        replace: Option<String>,
     ) -> Result<Self, ExhaustedCommandSpace> {
         let initial_args = match &action {
-            ExecAction::Command(args) => args.iter().map(|arg| arg.as_ref()).collect(),
+            ExecAction::Command(args) => args.iter().map(std::convert::AsRef::as_ref).collect(),
             ExecAction::Echo => vec![OsStr::new("echo")],
         };
 
@@ -366,6 +371,7 @@ impl CommandBuilderOptions {
             limiters,
             verbose: false,
             close_stdin: false,
+            replace,
         })
     }
 }
@@ -398,16 +404,38 @@ impl CommandBuilder<'_> {
         };
 
         let mut command = Command::new(entry_point);
-        command
-            .args(initial_args)
-            .args(&self.extra_args)
-            .env_clear()
-            .envs(&self.options.env);
+
+        if let Some(replace_str) = &self.options.replace {
+            // Replace all occurrences in initial args with the extra arg,
+            // Thanks to `MaxArgsCommandSizeLimiter`, we only process a single extra arg here.
+            let replacement = self.extra_args[0].to_string_lossy();
+            let initial_args: Vec<OsString> = initial_args
+                .iter()
+                .map(|arg| {
+                    let arg_str = arg.to_string_lossy();
+                    OsString::from(arg_str.replace(replace_str, &replacement))
+                })
+                .collect();
+
+            command
+                .args(&initial_args)
+                .env_clear()
+                .envs(&self.options.env);
+        } else {
+            // don't do any replacement
+            command
+                .args(initial_args)
+                .args(&self.extra_args)
+                .env_clear()
+                .envs(&self.options.env);
+        };
+
         if self.options.close_stdin {
             command.stdin(Stdio::null());
         }
+
         if self.options.verbose {
-            eprintln!("{:?}", command);
+            eprintln!("{command:?}");
         }
 
         match &self.options.action {
@@ -511,14 +539,14 @@ where
                     if let Some(Escape::Quote(q)) = &escape {
                         return Err(io::Error::new(
                             io::ErrorKind::InvalidInput,
-                            format!("Unterminated quote: {}", q),
+                            format!("Unterminated quote: {q}"),
                         ));
-                    } else if i == 0 {
-                        return Ok(None);
-                    } else {
-                        pending.clear();
-                        break;
                     }
+                    if i == 0 {
+                        return Ok(None);
+                    }
+                    pending.clear();
+                    break;
                 }
 
                 pending.resize(bytes_read, 0);
@@ -603,9 +631,8 @@ where
                     arg: String::from_utf8_lossy(bytes).into_owned().into(),
                     kind: ArgumentKind::HardTerminated,
                 });
-            } else {
-                break None;
             }
+            break None;
         })
     }
 }
@@ -622,9 +649,9 @@ impl Display for XargsError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
             XargsError::ArgumentTooLarge => write!(f, "Argument too large"),
-            XargsError::CommandExecution(e) => write!(f, "{}", e),
-            XargsError::Io(e) => write!(f, "{}", e),
-            XargsError::Untyped(s) => write!(f, "{}", s),
+            XargsError::CommandExecution(e) => write!(f, "{e}"),
+            XargsError::Io(e) => write!(f, "{e}"),
+            XargsError::Untyped(s) => write!(f, "{s}"),
         }
     }
 }
@@ -655,10 +682,33 @@ impl From<io::Error> for XargsError {
     }
 }
 
+struct InputProcessOptions {
+    exit_if_pass_char_limit: bool,
+    max_args: Option<usize>,
+    max_lines: Option<usize>,
+    no_run_if_empty: bool,
+}
+
+impl InputProcessOptions {
+    fn new(
+        exit_if_pass_char_limit: bool,
+        max_args: Option<usize>,
+        max_lines: Option<usize>,
+        no_run_if_empty: bool,
+    ) -> Self {
+        InputProcessOptions {
+            exit_if_pass_char_limit,
+            max_args,
+            max_lines,
+            no_run_if_empty,
+        }
+    }
+}
+
 fn process_input(
     builder_options: CommandBuilderOptions,
     mut args: Box<dyn ArgumentReader>,
-    options: &Options,
+    options: &InputProcessOptions,
 ) -> Result<CommandResult, XargsError> {
     let mut current_builder = CommandBuilder::new(&builder_options);
     let mut have_pending_command = false;
@@ -671,7 +721,8 @@ fn process_input(
                 && (options.max_args.is_some() || options.max_lines.is_some())
             {
                 return Err(XargsError::ArgumentTooLarge);
-            } else if have_pending_command {
+            }
+            if have_pending_command {
                 result.combine(current_builder.execute()?);
             }
 
@@ -692,12 +743,14 @@ fn process_input(
 }
 
 fn parse_delimiter(s: &str) -> Result<u8, String> {
-    if let Some(hex) = s.strip_prefix("\\x") {
-        u8::from_str_radix(hex, 16).map_err(|e| format!("Invalid hex sequence: {}", e))
-    } else if let Some(oct) = s.strip_prefix("\\0") {
-        u8::from_str_radix(oct, 8).map_err(|e| format!("Invalid octal sequence: {}", e))
-    } else if let Some(special) = s.strip_prefix('\\') {
-        match special {
+    match s.strip_prefix('\\') {
+        Some(hex) if hex.starts_with('x') => {
+            u8::from_str_radix(&hex[1..], 16).map_err(|e| format!("Invalid hex sequence: {e}"))
+        }
+        Some(oct) if oct.starts_with('0') => {
+            u8::from_str_radix(&oct[1..], 8).map_err(|e| format!("Invalid octal sequence: {e}"))
+        }
+        Some(special) => match special {
             "a" => Ok(b'\x07'),
             "b" => Ok(b'\x08'),
             "f" => Ok(b'\x0C'),
@@ -705,143 +758,62 @@ fn parse_delimiter(s: &str) -> Result<u8, String> {
             "r" => Ok(b'\r'),
             "t" => Ok(b'\t'),
             "v" => Ok(b'\x0B'),
-            "0" => Ok(b'\0'),
             "\\" => Ok(b'\\'),
-            _ => Err(format!("Invalid escape sequence: {}", s)),
-        }
-    } else {
-        let bytes = s.as_bytes();
-        if bytes.len() == 1 {
-            Ok(bytes[0])
-        } else {
-            Err("Delimiter must be one byte".to_owned())
-        }
+            "0" => Ok(b'\0'),
+            _ => Err(format!("Invalid escape sequence: \\{special}")),
+        },
+        None if s.len() == 1 => Ok(s.as_bytes()[0]),
+        None => Err("Delimiter must be one byte".to_owned()),
     }
 }
 
-fn validate_positive_usize(s: String) -> Result<(), String> {
+fn validate_positive_usize(s: &str) -> Result<usize, String> {
     match s.parse::<usize>() {
-        Ok(v) if v > 0 => Ok(()),
-        Ok(v) => Err(format!("Value must be > 0, not: {}", v)),
+        Ok(v) if v > 0 => Ok(v),
+        Ok(v) => Err(format!("Value must be > 0, not: {v}")),
         Err(e) => Err(e.to_string()),
     }
 }
 
-fn do_xargs(args: &[&str]) -> Result<CommandResult, XargsError> {
-    let matches = App::new("xargs")
-        .version(crate_version!())
-        .about("Run commands using arguments derived from standard input")
-        .settings(&[AppSettings::TrailingVarArg])
-        .arg(
-            Arg::with_name(options::COMMAND)
-                .takes_value(true)
-                .multiple(true)
-                .help("The command to run"),
-        )
-        .arg(
-            Arg::with_name(options::ARG_FILE)
-                .short("a")
-                .long(options::ARG_FILE)
-                .takes_value(true)
-                .help("Read arguments from the given file instead of stdin"),
-        )
-        .arg(
-            Arg::with_name(options::DELIMITER)
-                .short("d")
-                .long(options::DELIMITER)
-                .takes_value(true)
-                .validator(|s| parse_delimiter(&s).map(|_| ()))
-                .help("Use the given delimiter to split the input"),
-        )
-        .arg(
-            Arg::with_name(options::EXIT)
-                .short("x")
-                .long(options::EXIT)
-                .help(
-                    "Exit if the number of arguments allowed by -L or -n do not \
-                    fit into the number of allowed characters",
-                ),
-        )
-        .arg(
-            Arg::with_name(options::MAX_ARGS)
-                .short("n")
-                .takes_value(true)
-                .long(options::MAX_ARGS)
-                .validator(validate_positive_usize)
-                .help(
-                    "Set the max number of arguments read from stdin to be passed \
-                    to each command invocation (mutually exclusive with -L)",
-                ),
-        )
-        .arg(
-            Arg::with_name(options::MAX_LINES)
-                .short("L")
-                .takes_value(true)
-                .validator(validate_positive_usize)
-                .help(
-                    "Set the max number of lines from stdin to be passed to each \
-                    command invocation (mutually exclusive with -n)",
-                ),
-        )
-        .arg(
-            Arg::with_name(options::MAX_PROCS)
-                .short("P")
-                .takes_value(true)
-                .long(options::MAX_PROCS)
-                .help("Run up to this many commands in parallel [NOT IMPLEMENTED]"),
-        )
-        .arg(
-            Arg::with_name(options::NO_RUN_IF_EMPTY)
-                .short("r")
-                .long(options::NO_RUN_IF_EMPTY)
-                .help("If there are no input arguments, do not run the command at all"),
-        )
-        .arg(
-            Arg::with_name(options::NULL)
-                .short("0")
-                .long(options::NULL)
-                .help("Split the input by null terminators rather than whitespace"),
-        )
-        .arg(
-            Arg::with_name(options::SIZE)
-                .short("s")
-                .long(options::SIZE)
-                .takes_value(true)
-                .validator(validate_positive_usize)
-                .help(
-                    "Set the max number of characters to be passed to each \
-                    invocation",
-                ),
-        )
-        .arg(
-            Arg::with_name(options::VERBOSE)
-                .short("t")
-                .long(options::VERBOSE)
-                .help("Be verbose"),
-        )
-        .get_matches_from(args);
-
-    let options = Options {
-        arg_file: matches
-            .value_of(options::ARG_FILE)
-            .map(|value| value.to_owned()),
-        delimiter: matches
-            .value_of(options::DELIMITER)
-            .map(|value| parse_delimiter(value).unwrap()),
-        exit_if_pass_char_limit: matches.is_present(options::EXIT),
-        max_args: matches
-            .value_of(options::MAX_ARGS)
-            .map(|value| value.parse().unwrap()),
-        max_lines: matches
-            .value_of(options::MAX_LINES)
-            .map(|value| value.parse().unwrap()),
-        no_run_if_empty: matches.is_present(options::NO_RUN_IF_EMPTY),
-        null: matches.is_present(options::NULL),
-        size: matches
-            .value_of(options::SIZE)
-            .map(|value| value.parse().unwrap()),
-        verbose: matches.is_present(options::VERBOSE),
-    };
+fn normalize_options<'a>(
+    options: &'a Options,
+    matches: &'a clap::ArgMatches,
+) -> (Option<usize>, Option<usize>, &'a Option<String>, Option<u8>) {
+    let (max_args, max_lines, replace) =
+        match (options.max_args, options.max_lines, &options.replace) {
+            // These 3 options are mutually exclusive.
+            // But `max_args=1` and `replace` do not actually conflict, so no warning.
+            (None | Some(1), None, Some(_)) => {
+                // If `replace`, all matches in initial args should be replaced with extra args read from stdin.
+                // It is possible to have multiple matches and multiple extra args, and the Cartesian product is desired.
+                // To be specific, we process extra args one by one, and replace all matches with the same extra arg in each time.
+                (Some(1), None, &options.replace)
+            }
+            (Some(_), None, None) | (None, Some(_), None) | (None, None, None) => {
+                (options.max_args, options.max_lines, &None)
+            }
+            _ => {
+                eprintln!(
+                "WARNING: -L, -n and -I/-i are mutually exclusive, but more than one were given; \
+                only the last option will be used"
+            );
+                let lines_index = matches
+                    .indices_of(options::MAX_LINES)
+                    .and_then(|v| v.last());
+                let args_index = matches.indices_of(options::MAX_ARGS).and_then(|v| v.last());
+                let replace_index = [options::REPLACE, options::REPLACE_I]
+                    .iter()
+                    .flat_map(|o| matches.indices_of(o).and_then(|v| v.last()))
+                    .max();
+                if lines_index > args_index && lines_index > replace_index {
+                    (None, options.max_lines, &None)
+                } else if args_index > lines_index && args_index > replace_index {
+                    (options.max_args, None, &None)
+                } else {
+                    (Some(1), None, &options.replace)
+                }
+            }
+        };
 
     let delimiter = match (options.delimiter, options.null) {
         (Some(delimiter), true) => {
@@ -855,54 +827,191 @@ fn do_xargs(args: &[&str]) -> Result<CommandResult, XargsError> {
         }
         (Some(delimiter), false) => Some(delimiter),
         (None, true) => Some(b'\0'),
-        (None, false) => None,
+        // If `replace` and no delimiter specified, each line of stdin turns into a line of stdout,
+        // so the input should be split at newlines only.
+        (None, false) => replace.as_ref().map(|_| b'\n'),
     };
 
-    let action = match matches.values_of_os(options::COMMAND) {
+    (max_args, max_lines, replace, delimiter)
+}
+
+fn do_xargs(args: &[&str]) -> Result<CommandResult, XargsError> {
+    let matches = clap::Command::new("xargs")
+        .version(crate_version!())
+        .about("Run commands using arguments derived from standard input")
+        .arg(
+            Arg::new(options::COMMAND)
+                .help("The command to run")
+                .trailing_var_arg(true)
+                .num_args(0..)
+                .value_parser(clap::value_parser!(OsString)),
+        )
+        .arg(
+            Arg::new(options::ARG_FILE)
+                .short('a')
+                .long(options::ARG_FILE)
+                .help("Read arguments from the given file instead of stdin"),
+        )
+        .arg(
+            Arg::new(options::DELIMITER)
+                .short('d')
+                .long(options::DELIMITER)
+                .help("Use the given delimiter to split the input")
+                .value_parser(parse_delimiter),
+        )
+        .arg(
+            Arg::new(options::EXIT)
+                .short('x')
+                .long(options::EXIT)
+                .help(
+                    "Exit if the number of arguments allowed by -L or -n do not \
+                    fit into the number of allowed characters",
+                )
+                .action(ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new(options::MAX_ARGS)
+                .short('n')
+                .long(options::MAX_ARGS)
+                .help(
+                    "Set the max number of arguments read from stdin to be passed \
+                    to each command invocation (mutually exclusive with -L and -I/-i)",
+                )
+                .value_parser(validate_positive_usize),
+        )
+        .arg(
+            Arg::new(options::MAX_LINES)
+                .short('L')
+                .long(options::MAX_LINES)
+                .help(
+                    "Set the max number of lines from stdin to be passed to each \
+                    command invocation (mutually exclusive with -n and -I/-i)",
+                )
+                .value_parser(validate_positive_usize),
+        )
+        .arg(
+            Arg::new(options::MAX_PROCS)
+                .short('P')
+                .long(options::MAX_PROCS)
+                .help("Run up to this many commands in parallel [NOT IMPLEMENTED]")
+                .value_parser(clap::value_parser!(usize)),
+        )
+        .arg(
+            Arg::new(options::NO_RUN_IF_EMPTY)
+                .short('r')
+                .long(options::NO_RUN_IF_EMPTY)
+                .help("If there are no input arguments, do not run the command at all")
+                .action(ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new(options::NULL)
+                .short('0')
+                .long(options::NULL)
+                .help("Split the input by null terminators rather than whitespace")
+                .action(ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new(options::MAX_CHARS)
+                .short('s')
+                .long(options::MAX_CHARS)
+                .help(
+                    "Set the max number of characters to be passed to each \
+                    invocation",
+                )
+                .value_parser(validate_positive_usize),
+        )
+        .arg(
+            Arg::new(options::VERBOSE)
+                .short('t')
+                .long(options::VERBOSE)
+                .help("Be verbose")
+                .action(ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new(options::REPLACE)
+                .long(options::REPLACE)
+                .short('i')
+                .num_args(0..=1)
+                .require_equals(true)
+                .value_parser(clap::value_parser!(String))
+                .value_name("R")
+                .help("If R is specified, the same as -I R; otherwise, the same as -I {}"),
+        )
+        .arg(
+            Arg::new(options::REPLACE_I)
+                .short('I')
+                .num_args(1)
+                .value_name("R")
+                .help(
+                    "Replace R in initial arguments with names read from standard input; \
+                    also, the input is split at newlines only
+                    (mutually exclusive with -L and -n)",
+                )
+                .overrides_with(options::REPLACE)
+                .value_parser(clap::value_parser!(String)),
+        )
+        .try_get_matches_from(args);
+
+    let matches = match matches {
+        Ok(m) => m,
+        Err(e) => return Err(XargsError::from(e.to_string())),
+    };
+
+    let options = Options {
+        arg_file: matches
+            .get_one::<String>(options::ARG_FILE)
+            .map(std::borrow::ToOwned::to_owned),
+        delimiter: matches.get_one::<u8>(options::DELIMITER).copied(),
+        exit_if_pass_char_limit: matches.get_flag(options::EXIT),
+        max_args: matches.get_one::<usize>(options::MAX_ARGS).copied(),
+        max_chars: matches.get_one::<usize>(options::MAX_CHARS).copied(),
+        max_lines: matches.get_one::<usize>(options::MAX_LINES).copied(),
+        no_run_if_empty: matches.get_flag(options::NO_RUN_IF_EMPTY),
+        null: matches.get_flag(options::NULL),
+        replace: [options::REPLACE_I, options::REPLACE]
+            .iter()
+            .find_map(|&option| {
+                matches.contains_id(option).then(|| {
+                    matches
+                        .get_one::<String>(option)
+                        .map_or_else(|| "{}".to_string(), std::borrow::ToOwned::to_owned)
+                })
+            }),
+        verbose: matches.get_flag(options::VERBOSE),
+    };
+
+    let (max_args, max_lines, replace, delimiter) = normalize_options(&options, &matches);
+
+    let action = match matches.get_many::<OsString>(options::COMMAND) {
         Some(args) if args.len() > 0 => {
-            ExecAction::Command(args.map(|arg| arg.to_owned()).collect())
+            ExecAction::Command(args.map(std::borrow::ToOwned::to_owned).collect())
         }
         _ => ExecAction::Echo,
     };
-
     let env = std::env::vars_os().collect();
 
     let mut limiters = LimiterCollection::new();
-
-    match (options.max_args, options.max_lines) {
-        (Some(max_args), Some(max_lines)) => {
-            eprintln!(
-                "WARNING: Both --{} and -L were given; last option will be used",
-                options::MAX_ARGS,
-            );
-            if matches.indices_of(options::MAX_LINES).unwrap().last()
-                > matches.indices_of(options::MAX_ARGS).unwrap().last()
-            {
-                limiters.add(MaxLinesCommandSizeLimiter::new(max_lines));
-            } else {
-                limiters.add(MaxArgsCommandSizeLimiter::new(max_args));
-            }
-        }
-        (Some(max_args), None) => limiters.add(MaxArgsCommandSizeLimiter::new(max_args)),
-        (None, Some(max_lines)) => limiters.add(MaxLinesCommandSizeLimiter::new(max_lines)),
-        (None, None) => (),
+    if let Some(max_args) = max_args {
+        limiters.add(MaxArgsCommandSizeLimiter::new(max_args));
     }
-
-    if let Some(max_chars) = options.size {
+    if let Some(max_lines) = max_lines {
+        limiters.add(MaxLinesCommandSizeLimiter::new(max_lines));
+    }
+    if let Some(max_chars) = options.max_chars {
         limiters.add(MaxCharsCommandSizeLimiter::new(max_chars));
     }
-
     limiters.add(MaxCharsCommandSizeLimiter::new_system(&env));
 
-    let mut builder_options = CommandBuilderOptions::new(action, env, limiters).map_err(|_| {
-        "Base command and environment are too large to fit into one command execution"
-    })?;
+    let mut builder_options = CommandBuilderOptions::new(action, env, limiters, replace.clone())
+        .map_err(|_| {
+            "Base command and environment are too large to fit into one command execution"
+        })?;
 
     builder_options.verbose = options.verbose;
     builder_options.close_stdin = options.arg_file.is_none();
 
     let args_file: Box<dyn Read> = if let Some(path) = &options.arg_file {
-        Box::new(fs::File::open(path).map_err(|e| format!("Failed to open {}: {}", path, e))?)
+        Box::new(fs::File::open(path).map_err(|e| format!("Failed to open {path}: {e}"))?)
     } else {
         Box::new(io::stdin())
     };
@@ -913,16 +1022,26 @@ fn do_xargs(args: &[&str]) -> Result<CommandResult, XargsError> {
         Box::new(WhitespaceDelimitedArgumentReader::new(args_file))
     };
 
-    let result = process_input(builder_options, args, &options)?;
+    let result = process_input(
+        builder_options,
+        args,
+        &InputProcessOptions::new(
+            options.exit_if_pass_char_limit,
+            max_args,
+            max_lines,
+            options.no_run_if_empty,
+        ),
+    )?;
     Ok(result)
 }
 
+#[must_use]
 pub fn xargs_main(args: &[&str]) -> i32 {
     match do_xargs(args) {
         Ok(CommandResult::Success) => 0,
         Ok(CommandResult::Failure) => 123,
         Err(e) => {
-            eprintln!("Error: {}", e);
+            eprintln!("Error: {e}");
             if let XargsError::CommandExecution(cx) = e {
                 match cx {
                     CommandExecutionError::UrgentlyFailed => 124,
