@@ -21,6 +21,10 @@ fail_d = int(current["fail"]) - int(last["fail"])
 # Get an annotation to highlight changes
 print(f"::warning ::Changes from main: PASS {pass_d:+d} / SKIP {skip_d:+d} / FAIL {fail_d:+d}")
 
+# Check if there are no changes.
+if pass_d == 0:
+    print("::warning ::BFS tests No changes")
+
 # If results are worse fail the job to draw attention
 if pass_d < 0:
     sys.exit(1)
